package com.couchbase.demo;

import com.couchbase.client.ClusterManager;
import com.couchbase.client.CouchbaseClient;
import com.couchbase.client.CouchbaseConnectionFactoryBuilder;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.spy.memcached.ops.Operation;
import net.spy.memcached.ops.OperationQueueFactory;

/**
 *
 * @author ingenthr
 */
public class CouchbaseCheck {

  static int cores = Runtime.getRuntime().availableProcessors();
  static CouchbaseClient statClient[] = new CouchbaseClient[cores*4];
  static AtomicInteger clientIndex = new AtomicInteger(0);
  private static int numThreads = 8;

  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) throws URISyntaxException, IOException, InterruptedException, ExecutionException {
    CouchbaseCheck tester = new CouchbaseCheck();
    String uriArgs[] = {"localhost"};
    int runningTime = 45;
    int iterationsPerThread = 101;
    if (args.length >= 1) {
      runningTime = Integer.parseInt(args[0]);
      iterationsPerThread = Integer.parseInt(args[0]);
    }
    if (args.length >=2) {
      numThreads = Integer.parseInt(args[1]);
    }
    if (args.length >= 3) {
      uriArgs = new String[args.length - 2];
      System.arraycopy(args, 2, uriArgs, 0, args.length - 2);
    }

    List<URI> servers = new ArrayList<URI>();
    for (String arg : uriArgs) {
      servers.add(new URI("http://" + arg + ":8091/pools"));
    }


    System.out.println("Running for " + iterationsPerThread + " iterations with " + runningTime + " running time.");



    // Create a static client off this class to be grabbed elsewhere.
    // this is mainly so I can mess with it using CLI args
    CouchbaseConnectionFactoryBuilder cfb = new CouchbaseConnectionFactoryBuilder();
    //cfb.setObsPollInterval(60); // poll every 60ms during observe
    //cfb.setObsPollMax(10000);      // poll up to 10000 times for a total of 10 minutes

    OperationQueueFactory q = new OperationQueueFactory() {

      public BlockingQueue<Operation> create() {
        return new ArrayBlockingQueue<Operation>(65535);
      }
    };
    cfb.setOpTimeout(60000);
    cfb.setOpQueueFactory(q);
    cfb.setOpQueueMaxBlockTime(20000);

    for (int i=0; i<statClient.length; i++) {
      statClient[i] = new CouchbaseClient(cfb.buildCouchbaseConnection(servers, "cache", ""));
    }
    
    tester.testThem(servers, runningTime, iterationsPerThread);


  }

  public void testThem(List<URI> servers, int time, int iterations) throws IOException, InterruptedException, ExecutionException {

    // Tell spy to use the SunLogger
    Properties systemProperties = System.getProperties();
    systemProperties.put("net.spy.log.LoggerImpl", "net.spy.memcached.compat.log.SunLogger");
    System.setProperties(systemProperties);

    Logger.getLogger("net.spy.memcached").setLevel(Level.WARNING);
    Logger.getLogger("com.couchbase.demo").setLevel(Level.ALL);
    Logger.getLogger("com.couchbase.client.vbucket").setLevel(Level.ALL);
    Logger.getLogger("com.couchbase.client.vbucket.config").setLevel(Level.ALL);


    //get the top Logger
    Logger topLogger = java.util.logging.Logger.getLogger("");

    // Handler for console (reuse it if it already exists)
    Handler consoleHandler = null;
    //see if there is already a console handler
    for (Handler handler : topLogger.getHandlers()) {
      if (handler instanceof ConsoleHandler) {
        //found the console handler
        consoleHandler = handler;
        break;
      }
    }

    if (consoleHandler == null) {
      //there was no console handler found, create a new one
      consoleHandler = new ConsoleHandler();
      topLogger.addHandler(consoleHandler);
    }

    //set the console handler to fine:
    consoleHandler.setLevel(java.util.logging.Level.ALL);

    
    ClusterManager cman = new ClusterManager(servers, "Administrator", "password");
    long flushSleep = 1000;
    Logger.getLogger(CouchbaseCheck.class.getName()).log(Level.INFO,
      "Flushing and then waiting {0}ms.", flushSleep);
    cman.flushBucket("cache");
    Thread.sleep(flushSleep);

    
    ArrayList<Runnable> allWorkerThreads = new ArrayList<Runnable>();

    ExecutorService threadPool = Executors.newFixedThreadPool(numThreads+1);
    // TODO: make the threads run forever and report on threadpool shutdown

    Logger.getLogger(CouchbaseCheck.class.getName()).log(Level.CONFIG,
      "Starting {0} threads using {1} client objects across {2} CPU cores.",
      new Object[] {numThreads, statClient.length, cores});
    for (int i = 0; i < numThreads; i++) {
      SleepingTester workerThread = new SleepingTester(iterations);
      threadPool.submit(workerThread);
      allWorkerThreads.add(workerThread);
    }
    
    Noisemaker workerThread = new Noisemaker(10000);
    threadPool.submit(workerThread);
    allWorkerThreads.add(workerThread);
    
    



    Logger.getLogger(CouchbaseCheck.class.getName()).log(Level.INFO,
      "Started threads, will shut down in {0} seconds.", time);
    Thread.sleep(time * 1000);
    threadPool.shutdown();
    threadPool.awaitTermination(10, TimeUnit.SECONDS);
//        Map<SocketAddress, Map<String, String>> stats = cbc.getStats("timings");
//
//        StringBuilder statsString = new StringBuilder();
//
//        for ( Map.Entry<SocketAddress, Map<String,String>> node: stats.entrySet()) {
//             statsString.append("For node ").append(node.getKey()).append("\n");
//             for (Map.Entry<String,String> stat : node.getValue().entrySet()) {
//                 statsString.append("stat ").append(stat.getKey());
//                 statsString.append(" value ").append(stat.getValue()).append("\n");
//             }
//
//        }
//
//        Logger.getLogger(SleepingTester.class.getName()).log(Level.INFO,
//                "Server timing stats are: " + statsString);


    shutdownClients();
    
  }

  public static CouchbaseClient getTheClient() {
    return statClient[clientIndex.addAndGet(1)%8];
  }

  private void shutdownClients() {
    for (int i=0; i<statClient.length; i++) {
      statClient[i].shutdown(10, TimeUnit.SECONDS);
    }
  }

}
